"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.boot = exports.install = exports.update = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const yaml_1 = require("yaml");
const core_1 = require("./core");
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const node_toolkit_1 = require("@tech_query/node-toolkit");
const core_2 = require("create-es-pack/dist/core");
async function update(type) {
    const list = yaml_1.parse(await (await node_fetch_1.default(`https://raw.githubusercontent.com/hexojs/site/master/source/_data/${type}s.yml`)).text());
    await fs_extra_1.outputJSON(path_1.join(core_1.basePath, `data/${type}.json`), list, { spaces: 4 });
    console.info(`[ Update ]  ${list.length} ${type}s`);
}
exports.update = update;
/**
 * @return Names of installed packages
 */
async function install(path, type, packages) {
    const list = await fs_extra_1.readJSON(path_1.join(core_1.basePath, `data/${type}.json`)), NPM = [], Git = [], command_option = { stdio: 'inherit', cwd: path };
    for (const item of list) {
        if (!packages.find(key => RegExp(`^(hexo-(theme-)?)?${key}$`, 'i').test(item.name)))
            continue;
        if (item.link.includes('npmjs.com/'))
            NPM.push(item.name);
        else if (type === 'plugin')
            NPM.push(item.link);
        else
            Git.push(item);
    }
    if (NPM[0])
        await node_toolkit_1.spawn('npm', ['install', ...NPM], command_option);
    if (Git[0])
        for (const { link, name } of Git)
            await core_1.copyFromGit(link, path_1.join(path, `themes/${name}/`));
    return [...NPM, ...Git.map(({ name }) => name)];
}
exports.install = install;
async function addTheme(path, name, config) {
    name = (await install(path, 'theme', [name]))[0];
    if (!name)
        return console.error(`Theme "${name}" can't be found in Hexo offical index`);
    const root_path = path_1.join(path, 'themes', name);
    if (!fs_extra_1.existsSync(path_1.join(root_path, '_config.yml')))
        for (const file of fs_extra_1.readdirSync(root_path))
            if (/^_config\..*yml.*/.test(file)) {
                await fs_extra_1.move(path_1.join(root_path, file), path_1.join(root_path, '_config.yml'));
                break;
            }
    await fs_extra_1.remove(path_1.join(path, 'themes/landscape'));
    return config.replace(/^theme:.+/m, `theme: ${name}`);
}
/**
 * Boot a directory as a Hexo Wiki project
 */
async function boot({ path = '.', plugins = [], theme, pages = [], remote } = {}) {
    const command_option = { stdio: 'inherit', cwd: path }, config_path = path_1.join(path, '_config.yml');
    await node_toolkit_1.step('Hexo framework', async () => {
        await node_toolkit_1.ensureCommand('hexo');
        await core_1.copyFromGit('https://github.com/hexojs/hexo-starter.git', path);
    });
    let config, git;
    await node_toolkit_1.step('Git repository & branch', async () => {
        var _a;
        (config = (await fs_extra_1.readFile(config_path)) + ''),
            (git = await core_2.bootGit(path, remote));
        await git.raw(['checkout', '--orphan', 'hexo']);
        config = config.replace(/\ndeploy:[\s\S]+/, `
deploy:
  type: git
  repo: ${(_a = (await git.getRemotes(true))[0]) === null || _a === void 0 ? void 0 : _a.refs.push}
  branch: master`);
    });
    await node_toolkit_1.step('NPM package', () => core_1.setRoot(path, git));
    await node_toolkit_1.step('Hexo plugins', async () => {
        await node_toolkit_1.spawn('npm', ['install'], command_option);
        if (plugins[0])
            await install(path, 'plugin', plugins);
    });
    await node_toolkit_1.step('Hexo theme', async () => {
        if (theme)
            config = (await addTheme(path, theme, config)) || config;
        for (const name of pages) {
            const file = path_1.join(path, 'source', name, 'index.md');
            await fs_extra_1.outputFile(file, `---
type: ${name}
layout: ${name}
---\n`);
            console.info(`[ Create ]  ${file}`);
        }
        await fs_extra_1.outputFile(config_path, config);
    });
    await node_toolkit_1.step('Git commit', async () => {
        await git.add('.');
        await git.commit('[ Add ]  Framework of Hexo wiki');
    });
}
exports.boot = boot;
