"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.setPWA = exports.setRoot = exports.copyFromGit = exports.meta = exports.basePath = void 0;
const node_toolkit_1 = require("@tech_query/node-toolkit");
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const core_1 = require("create-es-pack/dist/core");
_a = node_toolkit_1.packageOf(node_toolkit_1.currentModulePath()), exports.basePath = _a.path, exports.meta = _a.meta;
const Git_file = [
    '.git/',
    '.gitignore',
    '.gitattributes',
    '.gitmodules',
    '.github/'
];
async function copyFromGit(URI, path = '', recurse) {
    await node_toolkit_1.spawn('git', ['clone', recurse ? '--recurse-submodules' : '', URI + '', path].filter(Boolean), { stdio: 'inherit' });
    for (const name of Git_file)
        await fs_extra_1.remove(path_1.join(path, name));
}
exports.copyFromGit = copyFromGit;
async function setRoot(path, git) {
    await core_1.copyFrom(path_1.join(exports.basePath, 'template'), path);
    core_1.fixIgnore('git', path);
    if (!node_toolkit_1.findFile(/ReadMe(\.(md|markdown))?/i, path))
        await core_1.setReadMe.call(exports.meta, path, git);
    const config = await core_1.setAuthor(path, git);
    (config.meta.name = node_toolkit_1.packageNameOf(path)), (config.meta.version = '1.0.0');
    await fs_extra_1.outputJSON(path_1.join(config.path, 'package.json'), config.meta);
}
exports.setRoot = setRoot;
function setPWA({ name, start_url, description, scope = '/', lang = 'en-US', dir = 'ltr', icons = [] } = {}) {
    return {
        manifest: {
            path: '/manifest.json',
            body: {
                name,
                short_name: name,
                start_url,
                description,
                scope,
                display: 'standalone',
                orientation: 'any',
                lang,
                dir,
                theme_color: 'rgba(0,0,0,0.5)',
                background_color: 'transparent',
                icons
            }
        },
        serviceWorker: {
            globDirectory: 'public/',
            globPatterns: ['**/*.{html,css,js,ico,gif,png}'],
            swDest: 'public/sw.js',
            importWorkboxFrom: 'disabled',
            importScripts: [
                'https://cdn.jsdelivr.net/npm/workbox-sw@4.3.1/build/workbox-sw.min.js'
            ]
        }
    };
}
exports.setPWA = setPWA;
