#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const commander_jsx_1 = require("commander-jsx");
const core_1 = require("./core");
const command_1 = require("./command");
const package_pattern = /^([@\-\w]+,?)+$/, repository_pattern = /^(git|ssh|https?):\/\/\S+\.git$/;
async function create({ plugins, theme = 'Wikitten', pages, remote }, path) {
    console.time('Boot Wiki');
    await command_1.boot({ path, plugins, theme, pages, remote });
    console.info('--------------------');
    console.timeEnd('Boot Wiki');
    console.info('\n[ Document ]  https://tech-query.me/create-hexo-wiki/\n');
}
async function refresh() {
    console.time('Update');
    await command_1.update('plugin');
    await command_1.update('theme');
    console.info('---------');
    console.timeEnd('Update');
}
commander_jsx_1.Command.execute(commander_jsx_1.createCommand("create-hexo-wiki", { parameters: "<command> [options]", version: core_1.meta.version, description: core_1.meta.description },
    commander_jsx_1.createCommand("create", { parameters: "<path>", description: "Create a Hexo Wiki project", options: {
            plugins: {
                shortcut: 'p',
                parameters: '<list>',
                pattern: package_pattern,
                description: 'Plugins to install (comma separated)'
            },
            theme: {
                shortcut: 't',
                parameters: '<name>',
                pattern: package_pattern,
                description: 'A theme to install'
            },
            pages: {
                shortcut: 'P',
                parameters: '<list>',
                description: 'Required pages of installed theme'
            },
            remote: {
                shortcut: 'r',
                parameters: '<URL>',
                pattern: repository_pattern,
                description: 'Git URL of a Remote repository'
            }
        }, executor: create }),
    commander_jsx_1.createCommand("update", { description: "Update list of Plugins & Themes", executor: refresh })), process.argv.slice(2));
